package de.hft.stuttgart.citydoctor2.utils;

import de.hft.stuttgart.citydoctor2.datastructure.CityDoctorModel;
import de.hft.stuttgart.citydoctor2.zip.CityGmlZipArchive;
import de.hft.stuttgart.citydoctor2.zip.CityGmlZipEntry;
import de.hft.stuttgart.citydoctor2.zip.ErroneousEntry;
import org.apache.commons.io.FileUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

public class ArchivePacker {

    private static final Logger logger = LogManager.getLogger(ArchivePacker.class);

    private ArchivePacker(){}

    public static void packArchive(String targetPath, CityGmlZipArchive archive){
        if (!targetPath.endsWith(".zip")){
            targetPath = targetPath.concat(".zip");
        }
        Path tmpDir = null;
        try{
            tmpDir = Files.createTempDirectory("zipTmp");
            tmpDir.toFile().deleteOnExit();
            for (CityGmlZipEntry entry : archive.getEntries()) {
                if (entry instanceof ErroneousEntry){
                    continue;
                }
                CityDoctorModel model = entry.getModel();
                String filename = tmpDir.toString() + File.separator + entry.getFileName();
                model.saveAs(filename, entry.isValidated());
            }
            zipDirectory(targetPath, tmpDir.toString());

        }catch (Exception e){
            logger.error(e);
        } finally {
            if (tmpDir != null) {
                try {
                    FileUtils.deleteDirectory(tmpDir.toFile());
                } catch (IOException e) {
                    logger.error(e);
                }
            }
        }
    }

    private static void zipDirectory(String targetPath, String sourcePath){
        File directory = new File(sourcePath);
        List<String> fileList = new ArrayList<>(Arrays.asList(directory.list()));

        byte[] buffer = new byte[1024];
        FileOutputStream fos;
        ZipOutputStream zos = null;
        try {
            fos = new FileOutputStream(targetPath);
            zos = new ZipOutputStream(fos);
            FileInputStream in = null;
            for (String file : fileList){
                ZipEntry ze = new ZipEntry(file);
                zos.putNextEntry(ze);
                try {
                    Path sourceFile = Path.of(sourcePath+File.separator+file);
                    in = new FileInputStream(sourceFile.toAbsolutePath().toString());
                    int len;
                    while ((len = in.read(buffer)) > 0) {
                        zos.write(buffer, 0, len);
                    }
                }finally {
                    if (in != null) {
                        in.close();
                    }
                }
            }
            zos.closeEntry();
            logger.info("Successfully created zip-archive");
        } catch (IOException e) {
            logger.error(e);
        } finally {
            if (zos != null) {
                try {
                    zos.close();
                } catch (IOException e) {
                    logger.error(e);
                }
            }
        }
    }

}
